﻿using BankContracts.BindingModels;
using BankContracts.BusinessLogicContracts;
using BankContracts.SearchModels;
using BankContracts.StoragesContracts;
using BankContracts.ViewModels;
using Microsoft.Extensions.Logging;
using System.Text.RegularExpressions;

namespace BankBusinessLogics.BusinessLogic
{
    public class ClientLogic : IClientLogic
    {
        private readonly ILogger _logger;
        private readonly IClientStorage _clientStorage;
        public ClientLogic(ILogger<ClientLogic> logger, IClientStorage clientStorage)
        {
            _logger = logger;
            _clientStorage = clientStorage;
        }

        private void CheckModel(ClientBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.PhoneNumber))
            {
                throw new ArgumentNullException(nameof(model.PhoneNumber), "Нет логина клиента");
            }
            if (string.IsNullOrEmpty(model.Password))
            {
                throw new ArgumentNullException(nameof(model.Password), "Нет пароля клиента");
            }
            if (model.PhoneNumber.Length is < 11)
            {
                throw new ArgumentException(nameof(model.PhoneNumber), "Длина номера телефона должна быть 11 цифр");
            }

            if (model.Password.Length < 5)
            {
                throw new ArgumentException(nameof(model.Password),
                    "Пароль пользователя должен быть не менее 5 символов");
            }
            if (!Regex.IsMatch(model.Password, "^[0-9]+"))
            {
                throw new ArgumentException(nameof(model.Password),
                    "Пароль пользователя должен содержать хотя бы одну цифру");
            }
            _logger.LogDebug("{level} Проверка логина пользователя на уникальность {@Client}", model);
            var element = _clientStorage.GetElement(new ClientSearchModel
            {
                PhoneNumber = model.PhoneNumber,
            });
            if (element != null && element.Id != model.Id)
            {
                _logger.LogWarning("С номером {PhoneNumber}, уже есть пользователь: {@ExistClient}", model.PhoneNumber, element);
                throw new InvalidOperationException($"Клиент с таким номером телефона уже есть");
            }
        }

        public bool Create(ClientBindingModel model)
        {
            try
            {
                CheckModel(model);
                var result = _clientStorage.Insert(model);
                if (result == null)
                {
                    throw new ArgumentNullException($"Клиент не создался");
                }
                _logger.LogInformation("Создана сущность: {@ClientViewModel}", result);
                return true;
            }
            catch (Exception e)
            {
                _logger.LogError(e, "Ошибка при попытки создать элемент по {@ClientBindingModel} модели", model);
                throw;
            }
        }

        public ClientViewModel ReadElement(ClientSearchModel model)
        {
            try
            {
                var result = _clientStorage.GetElement(model);
                if (result == null)
                {
                    throw new ArgumentNullException($"Результат получения элемента с id {model.Id} оказался нулевым");
                }
                _logger.LogInformation("Извлечение элемента {@ClientViewModel} c клиента по модели: {@ClientSearchModel}", result, model);
                return result;
            }
            catch (Exception e)
            {
                _logger.LogError(e, "Произошла ошибка при попытки получить элемент по модели: {@ClientSearchModel}", model);
                throw;
            }
        }
    }
}
